/*
** Vinco_Voltmeter.c
**
** Copyright  2011 Future Technology Devices International Limited
**
** C Source file for Vinculum II test applications
** Main module
**
** Author: FTDI - P. Rybak
** Project: Vinculum II
** Module: Vinculum II / Vinco Applications
** Requires: VOS, DEVMAN, IOMUX, UART, GPIO, SPIMaster, PWM, STDIO
** Comments:
**
** History:
**  1  Initial version
**
*/

#include "vos.h"
#include "devman.h"
#include "IOMUX.h"

#include "UART.h"
#include "GPIO.h"
#include "SPIMaster.h"
#include "PWM.h"
#include "ADC_MCP3008.h"

#include "stdio.h"

#include "Vinco_Voltmeter.h"
#include "GSGCdef.h"

/* VOS TCB pointers for threads */
vos_tcb_t        *tcbADC_thread;
vos_tcb_t        *tcbPWM_thread;

// UART IOCTL request block
common_ioctl_cb_t uart_iocb;
// GPIO IOCTL request block
gpio_ioctl_cb_t gpio_iocb;
// SPI Master request block
common_ioctl_cb_t spim_iocb;
// ADC request block
adc_mcp3008_ioctl_cb_t adc_iocb;
adc_mcp3008_ioctl_cb_attach_t adc_attach_info;

// VOS handles for devices to be used
VOS_HANDLE hUart, hSPIm, hGpio_A, hGpio_B, hPwm, hAdc;

unsigned char ADC_buff[16];

// forward declarations
void ADC_thread(void);
void PWM_thread(void);

/* main
*
* This is the main application called by VOS at startup. It is responsible
* for device and thread initialisation.
*/
void main(void)
{
    // GPIO context structure
    gpio_context_t gpio_Ctx;
    // UART context structure
    uart_context_t uart_Ctx;
    // SPI Master context structure
    spimaster_context_t spim_Ctx;

    // call VOS initialisation routines
    vos_init(10, VOS_TICK_INTERVAL, NUMBER_OF_DEVICES);
    vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);

    // Route IO signals
        // GPIO port A bit 0 to pin 39
        vos_iomux_define_output(39,IOMUX_OUT_GPIO_PORT_A_0); // LED1#
        vos_iocell_set_config(39, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_UP_75K);
        // GPIO port A bit 1 to pin 40
        vos_iomux_define_output(40,IOMUX_OUT_GPIO_PORT_A_1); // LED2#
        vos_iocell_set_config(40, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_UP_75K);
        // GPIO port A bit 2 to pin 41
        vos_iomux_define_output(41,IOMUX_OUT_GPIO_PORT_A_2); // PWREN#
        vos_iocell_set_config(41, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_UP_75K);

        // GPIO port A bit 6 to pin 55
        vos_iomux_define_output(55,IOMUX_OUT_GPIO_PORT_A_6); // Display RESET#
        vos_iocell_set_config(55, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_UP_75K);

        //---------------------------------------
        // Configure GPIO Port B as high-impedance inputs
        // GPIO port B bit 0 to pin 43
        vos_iomux_define_input(43,IOMUX_IN_GPIO_PORT_B_0); // AIN0
        vos_iocell_set_config(43, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_NONE);
        // GPIO port B bit 1 to pin 44
        vos_iomux_define_input(44,IOMUX_IN_GPIO_PORT_B_1); // AIN1
        vos_iocell_set_config(44, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_NONE);
        // GPIO port B bit 2 to pin 45
        vos_iomux_define_input(45,IOMUX_IN_GPIO_PORT_B_2); // AIN2
        vos_iocell_set_config(45, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_NONE);
        // GPIO port B bit 3 to pin 46
        vos_iomux_define_input(46,IOMUX_IN_GPIO_PORT_B_3); // AIN3
        vos_iocell_set_config(46, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_NONE);
        // GPIO port B bit 4 to pin 47
        vos_iomux_define_input(47,IOMUX_IN_GPIO_PORT_B_4); // AIN4
        vos_iocell_set_config(47, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_NONE);
        // GPIO port B bit 5 to pin 48
        vos_iomux_define_input(48,IOMUX_IN_GPIO_PORT_B_5); // AIN5
        vos_iocell_set_config(48, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_NONE);
        // GPIO port B bit 6 to pin 49
        vos_iomux_define_input(49,IOMUX_IN_GPIO_PORT_B_6); // AIN6
        vos_iocell_set_config(49, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_NONE);
        // GPIO port B bit 7 to pin 50
        vos_iomux_define_input(50,IOMUX_IN_GPIO_PORT_B_7); // AIN7
        vos_iocell_set_config(50, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_NONE);

        // UART to Vinculo board pins
        vos_iomux_define_output(51,IOMUX_OUT_UART_TXD); // UART Tx
        vos_iocell_set_config(51, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_UP_75K);
        vos_iomux_define_input(52,IOMUX_IN_UART_RXD); // UART Rx
        vos_iocell_set_config(52, VOS_IOCELL_DRIVE_CURRENT_4MA,
                                  VOS_IOCELL_TRIGGER_NORMAL,
                                  VOS_IOCELL_SLEW_RATE_FAST,
                                  VOS_IOCELL_PULL_UP_75K);

    // PWM Outputs
    // All PWM outputs are connected to ADC inputs
    // through 1k/1uF R/C filter.
    vos_iomux_define_output(24, IOMUX_OUT_PWM_0); // PWM_0
    vos_iocell_set_config(24, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);
    vos_iomux_define_output(25, IOMUX_OUT_PWM_1); // PWM_1
    vos_iocell_set_config(25, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);
    vos_iomux_define_output(26, IOMUX_OUT_PWM_2); // PWM_2
    vos_iocell_set_config(26, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);
    vos_iomux_define_output(27, IOMUX_OUT_PWM_3); // PWM_3
    vos_iocell_set_config(27, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);
    vos_iomux_define_output(28, IOMUX_OUT_PWM_4); // PWM_4
    vos_iocell_set_config(28, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);
    vos_iomux_define_output(29, IOMUX_OUT_PWM_5); // PWM_5
    vos_iocell_set_config(29, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);
    vos_iomux_define_output(31, IOMUX_OUT_PWM_6); // PWM_6
    vos_iocell_set_config(31, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);
    vos_iomux_define_output(32, IOMUX_OUT_PWM_7); // PWM_7
    vos_iocell_set_config(32, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);


    // SPI Master to Vinculo board pins
    vos_iomux_define_output(19,IOMUX_OUT_SPI_MASTER_CLK); // SPI_M CLK
    vos_iocell_set_config(19, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);
    vos_iomux_define_output(20,IOMUX_OUT_SPI_MASTER_MOSI); // SPI_M MOSI
    vos_iocell_set_config(20, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);
    vos_iomux_define_input(22,IOMUX_IN_SPI_MASTER_MISO); // SPI_M MISO
    vos_iocell_set_config(22, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);
    vos_iomux_define_output(23,IOMUX_OUT_SPI_MASTER_CS_0); // SPI_M CS0
    vos_iocell_set_config(23, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);
    // CS# signal for onboard ADC
    vos_iomux_define_output(61,IOMUX_OUT_SPI_MASTER_CS_1); // SPI_M CS1
    vos_iocell_set_config(61, VOS_IOCELL_DRIVE_CURRENT_4MA,
                              VOS_IOCELL_TRIGGER_NORMAL,
                              VOS_IOCELL_SLEW_RATE_FAST,
                              VOS_IOCELL_PULL_UP_75K);


    // initialise device drivers
    uart_Ctx.buffer_size = VOS_BUFFER_SIZE_128_BYTES;
    uart_init(VOS_DEV_UART,&uart_Ctx);

    spim_Ctx.buffer_size = VOS_BUFFER_SIZE_64_BYTES;
    spimaster_init(VOS_DEV_SPIM,&spim_Ctx);

    gpio_Ctx.port_identifier = GPIO_PORT_A;
    gpio_init(VOS_DEV_GPIO_A,&gpio_Ctx);

    gpio_Ctx.port_identifier = GPIO_PORT_B;
    gpio_init(VOS_DEV_GPIO_B,&gpio_Ctx);

    pwm_init(VOS_DEV_PWM);
    // enable PWM interrupts
    vos_enable_interrupts(VOS_PWM_TOP_INT_IEN);

    adc_mcp3008_init(VOS_DEV_ADC);

    // create threads for firmware application (no parameters)
    tcbADC_thread = vos_create_thread(29, SIZEOF_FIRMWARE_TASK_MEMORY, &ADC_thread, 0);
    tcbPWM_thread = vos_create_thread(29, SIZEOF_FIRMWARE_TASK_MEMORY, &PWM_thread, 0);
    // start VOS scheduler
    vos_start_scheduler();

main_loop:
    goto main_loop;
}
//---------------------------------------------------------
// Display functions
//---------------------------------------------------------
unsigned char Get_ACK(void)
{
    unsigned char i;

    // Check if ACK from display received
    uart_iocb.ioctl_code = VOS_IOCTL_COMMON_GET_RX_QUEUE_STATUS;
    vos_dev_ioctl(hUart, &uart_iocb);
    i = uart_iocb.get.queue_stat; // How many bytes to read?
    if(i>0)
    {
        // Read data from UART
        vos_dev_read(hUart, ADC_buff, (i & 0xFF), NULL);
        if(ADC_buff[0] == ACK)
        {
            return 1;
        }
        else
        {
            return 0;
        }
    }
}
//---------------------------------------------------------
void Display_Init(void)
{
    unsigned char j;

    // Toggle display RESET# pin
    j = 0xFF&(~DISPRST_n);
    vos_dev_write(hGpio_A, &j, 1, NULL);
    vos_delay_msecs(10);
    j = 0xFF;
    vos_dev_write(hGpio_A, &j, 1, NULL);
    vos_delay_msecs(1000);

    // Durring power-up display may send garbage.
    // Perform a dummy read from the display.
    do
    {
        // Get Rx queue
        uart_iocb.ioctl_code = VOS_IOCTL_COMMON_GET_RX_QUEUE_STATUS;
        vos_dev_ioctl(hUart, &uart_iocb);
        j = uart_iocb.get.queue_stat; // How much data to read?
        if(j > 0)
        {
            // Read data from UART
            vos_dev_read(hUart, ADC_buff, (j & 0xFF), NULL);
        }
    }while(j > 0);

    // Try to synchronize baudrate with display.
    do
    {
        // Turn on LED2#
        j=(~LED2);
        vos_dev_write(hGpio_A, &j, 1, NULL);
        // Send AUTOBAUD command to the display
        ADC_buff[0] = GSGC_AUTOBAUD;
        vos_dev_write(hUart, ADC_buff, 1, NULL);
        vos_delay_msecs(100);
        // Turn off LED2#
        j=0xFF;
        vos_dev_write(hGpio_A, &j, 1, NULL);
        // Check if ACK from display received
        if(Get_ACK())
        {
            break;
        }
    }while(1);

    // Setup the display
    ADC_buff[0] = GSGC_SETOPAQUE;
    ADC_buff[1] = 1;
    vos_dev_write(hUart, ADC_buff, 2, NULL);
    vos_delay_msecs(50);
    // Wait for ACK from display
    do
    {
        if(Get_ACK())
        {
            break;
        }
    }while(1);

    ADC_buff[0] = GSGC_DISPCONT;
    ADC_buff[1] = 2; // Mode = (adjust contrast)
    ADC_buff[2] = 0x0F; // Contrast value (0x0F..0x00)
    vos_dev_write(hUart, ADC_buff, 3, NULL);
    vos_delay_msecs(50);
    // Wait for ACK from display
    do
    {
        if(Get_ACK())
        {
            break;
        }
    }while(1);
}
//---------------------------------------------------------
// ADC thread created by the main application.
// It will read all 8 ADC inputs and display the readings
// on uOLED display. Color of the font and background
// will change according to the voltage present on ADC inputs.
void ADC_thread(void)
{
    // general purpose variables
    unsigned short rgb[8]; // To store voltage reading for each ADC channel
    unsigned short ADC_Val = 0; // Raw ADC reading
    unsigned short i = 0;
    unsigned char j = 0;

    // Open GPIO device port A
    hGpio_A = vos_dev_open(VOS_DEV_GPIO_A);
    gpio_iocb.ioctl_code = VOS_IOCTL_GPIO_SET_MASK;
    gpio_iocb.value = (LED1|LED2|PWREN_n|DISPRST_n);    // set bits 6, 2..0 as outputs, all other as inputs
    vos_dev_ioctl(hGpio_A, &gpio_iocb);

    // Open GPIO device port B
    hGpio_B = vos_dev_open(VOS_DEV_GPIO_B);
    gpio_iocb.ioctl_code = VOS_IOCTL_GPIO_SET_MASK;
    gpio_iocb.value = 0x00;    // set all as input
    vos_dev_ioctl(hGpio_B, &gpio_iocb);

    // Open UART device
    hUart = vos_dev_open(VOS_DEV_UART);

    // Enable DMA for UART driver
    uart_iocb.ioctl_code = VOS_IOCTL_COMMON_ENABLE_DMA;
    vos_dev_ioctl(hUart,&uart_iocb);

    // Setup transmision paramemters
    //set baud rate
    uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_BAUD_RATE;
    uart_iocb.set.uart_baud_rate = 230400;
    vos_dev_ioctl(hUart,&uart_iocb);
    //set flow control
    uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_FLOW_CONTROL;
    uart_iocb.set.param = UART_FLOW_NONE;
    vos_dev_ioctl(hUart,&uart_iocb);
    //set data bits
    uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_DATA_BITS;
    uart_iocb.set.param = UART_DATA_BITS_8;
    vos_dev_ioctl(hUart,&uart_iocb);
    // set stop bits
    uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_STOP_BITS;
    uart_iocb.set.param = UART_STOP_BITS_1;
    vos_dev_ioctl(hUart,&uart_iocb);
    // set parity
    uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_PARITY;
    uart_iocb.set.param = UART_PARITY_NONE;
    vos_dev_ioctl(hUart,&uart_iocb);

    // Attach UART to stdio driver
    // Formatted strings with voltage readings will be sent
    // to UART using 'printf' function.
    stdioAttach(hUart);
    //-------------------------------------------
    // Setup SPI Master
    // open SPI Master and get a handle
    hSPIm = vos_dev_open(VOS_DEV_SPIM);
    // enable DMA
    spim_iocb.ioctl_code = VOS_IOCTL_COMMON_ENABLE_DMA;
    vos_dev_ioctl(hSPIm,&spim_iocb);
    // set clock phase
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SCK_CPHA;
    spim_iocb.set.param = SPI_MASTER_SCK_CPHA_1; // Data will be clocked in to ADC on rising edge
    vos_dev_ioctl(hSPIm,&spim_iocb);
    // set clock polarity
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SCK_CPOL;
    spim_iocb.set.param = SPI_MASTER_SCK_CPOL_1; // Clock will be high in idle
    vos_dev_ioctl(hSPIm,&spim_iocb);
    // set data order
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_DATA_ORDER;
    spim_iocb.set.param = SPI_MASTER_DATA_ORDER_MSB; // MSB first
    vos_dev_ioctl(hSPIm,&spim_iocb);
    // set clock rate
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SET_SCK_FREQUENCY;
    spim_iocb.set.spi_master_sck_freq = 3000000;
    vos_dev_ioctl(hSPIm,&spim_iocb);
    // Set data delay
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SET_DATA_DELAY;
    spim_iocb.set.param = 0;
    vos_dev_ioctl(hSPIm,&spim_iocb);
    // set initial state of chip select 0 pin
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SS_0;
    spim_iocb.set.param = SPI_MASTER_SS_DISABLE;
    vos_dev_ioctl(hSPIm,&spim_iocb);
    // set initial state of chip select 1 pin
    spim_iocb.ioctl_code = VOS_IOCTL_SPI_MASTER_SS_1;
    spim_iocb.set.param = SPI_MASTER_SS_DISABLE;
    vos_dev_ioctl(hSPIm,&spim_iocb);

    // open ADC driver
    hAdc = vos_dev_open(VOS_DEV_ADC);

    // attach ADC driver to SPI master
    adc_iocb.ioctl_code = VOS_IOCTL_ADC_MCP3008_ATTACH;
    adc_attach_info.spi_master_handle = hSPIm;
    adc_attach_info.chip_select_identifier = ADC_MCP3008_CHIP_SELECT_1;
    adc_iocb.attach_info =  &adc_attach_info;
    vos_dev_ioctl(hAdc,&adc_iocb);

    //-------------------------------------------
    // Initialise display
    Display_Init();
    //-------------------------------------------
    // loop
    do
    {
        // Read all 8 channels of ADC in single-ended mode.
        for(j=0;j<8;j++)
        {
            adc_iocb.ioctl_code = VOS_IOCTL_ADC_MCP3008_READ_CHANNEL;
            adc_iocb.mode = ADC_MCP3008_MODE_SINGLE_ENDED;
            adc_iocb.channel = j;
            vos_dev_ioctl(hAdc,&adc_iocb);
            // Convert the value to mV
            ADC_Val = ((adc_iocb.value*32) / 10);
            rgb[j] = ADC_Val;
        }

        //-------------------------------------------
        // Display voltage readings.
        for(j=0;j<=7;j++)
        {
            // Set font color and size
            // font or background color have 16-bit value as follows:
            //       [R4R3R2R1R0 G5G4G3G2G1G0 B4B3B2B1B0]
            // It is possible to display 65000 colors
            ADC_Val = ((rgb[0]>>5) & 0x1F); // RED value
            ADC_Val = (ADC_Val << 6) | ((rgb[1]>>4) & 0x3F); // GREEN value
            ADC_Val = (ADC_Val << 5) | ((rgb[2]>>5) & 0x1F); // BLUE value
            ADC_buff[0]= GSGC_STRINGTXT; // Command
            ADC_buff[1]= 3;   // Cloumn
            ADC_buff[2]= j+1;   // Row
            ADC_buff[3]= FONT3; // Font
            ADC_buff[4]= ((ADC_Val >> 8) & 0xFF); // Colour MSB
            ADC_buff[5]= (ADC_Val & 0xFF); // Colour LSB
            vos_dev_write(hUart, ADC_buff, 6, NULL);
            // Display ADC readings
            if(rgb[j] < 10)
            {
                printf(" Ch-%u: 0.00%uV", (j & 0x0F), rgb[j]);
            }
            else if(rgb[j] < 100)
            {
                printf(" Ch-%u: 0.0%uV", (j & 0x0F), rgb[j]);
            }
            else if(rgb[j] < 1000)
            {
                printf(" Ch-%u: 0.%uV", (j & 0x0F), rgb[j]);
            }
            else // ADC_Val > 1000
            {
                i = (rgb[j] % 1000);
                if(i < 10)
                {
                    printf(" Ch-%u: %u.00%uV", (j & 0x0F), ((rgb[j] / 1000) & 0xFFFF),(i & 0xFFFF));
                }
                else if(i < 100)
                {
                    printf(" Ch-%u: %u.0%uV", (j & 0x0F), ((rgb[j] / 1000) & 0xFFFF),(i & 0xFFFF));
                }
                else // ADC_Val % 1000 > 100
                {
                    printf(" Ch-%u: %u.%uV", (j & 0x0F), ((rgb[j] / 1000) & 0xFFFF),(i & 0xFFFF));
                }
            }
            ADC_buff[0]= '\0'; // String termination
            vos_dev_write(hUart, ADC_buff, 1, NULL);
	        // Wait for ACK
	        do
 	       {
  	          if(Get_ACK())
   		      {
      	          break;
       	      }
           }while(1);
        }
        //-------------------------------------------
        // Set background color
        ADC_Val = ((rgb[3]>>5) & 0x1F); // RED value
        ADC_Val = (ADC_Val << 6) | ((rgb[4]>>4) & 0x3F); // GREEN value
        ADC_Val = (ADC_Val << 5) | ((rgb[5]>>5) & 0x1F); // BLUE value
        ADC_buff[0]= GSGC_BACKGND_REPLACE; // Command
        ADC_buff[1]= ((ADC_Val >> 8) & 0xFF); // Color MSB
        ADC_buff[2]= (ADC_Val & 0xFF); // Color LSB
        vos_dev_write(hUart, ADC_buff, 3, NULL);
        // Wait for ACK
        do
        {
            if(Get_ACK())
            {
                break;
            }
        }while(1);

    } while(1);
}
//-------------------------------------------
void PWM_thread(void)
{
    pwm_ioctl_cb_t pwm_iocb;
    unsigned short count0 = 0x0001;
    unsigned short count1 = 0x002A;
    unsigned short count2 = 0x0055;
    unsigned short count3 = 0x00AA;
    unsigned char direction0 = 1; // up
    unsigned char direction1 = 0; // down
    unsigned char direction2 = 1; // up
    unsigned char direction3 = 0; // down

    // open pwm and get a handle
    hPwm = vos_dev_open(VOS_DEV_PWM);

    // set prescaler value
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_PRESCALER_VALUE;
    pwm_iocb.count.prescaler = 0xF0;
    vos_dev_ioctl(hPwm, &pwm_iocb);

    // set counter value
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COUNTER_VALUE;
    pwm_iocb.count.value = 0x00B0;
    vos_dev_ioctl(hPwm, &pwm_iocb);

    // set comparator 0 value
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COMPARATOR_VALUE;
    pwm_iocb.identifier.comparator_number = COMPARATOR_0;
    pwm_iocb.comparator.value = count0;
    vos_dev_ioctl(hPwm, &pwm_iocb);
    // set comparator 1 value
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COMPARATOR_VALUE;
    pwm_iocb.identifier.comparator_number = COMPARATOR_1;
    pwm_iocb.comparator.value = count1;
    vos_dev_ioctl(hPwm, &pwm_iocb);
    // set comparator 2 value
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COMPARATOR_VALUE;
    pwm_iocb.identifier.comparator_number = COMPARATOR_2;
    pwm_iocb.comparator.value = count2;
    vos_dev_ioctl(hPwm, &pwm_iocb);
    // set comparator 3 value
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COMPARATOR_VALUE;
    pwm_iocb.identifier.comparator_number = COMPARATOR_3;
    pwm_iocb.comparator.value = count3;
    vos_dev_ioctl(hPwm, &pwm_iocb);

    // enable comparator 0 for PWM_0 and PWM_4
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_OUTPUT_TOGGLE_ENABLES;
    pwm_iocb.identifier.pwm_number = PWM_0;
    pwm_iocb.output.enable_mask = MASK_COMPARATOR_0;
    vos_dev_ioctl(hPwm, &pwm_iocb);
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_OUTPUT_TOGGLE_ENABLES;
    pwm_iocb.identifier.pwm_number = PWM_4;
    pwm_iocb.output.enable_mask = MASK_COMPARATOR_0;
    vos_dev_ioctl(hPwm, &pwm_iocb);
    // enable comparator 1 for PWM_1 and PWM_5
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_OUTPUT_TOGGLE_ENABLES;
    pwm_iocb.identifier.pwm_number = PWM_1;
    pwm_iocb.output.enable_mask = MASK_COMPARATOR_1;
    vos_dev_ioctl(hPwm, &pwm_iocb);
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_OUTPUT_TOGGLE_ENABLES;
    pwm_iocb.identifier.pwm_number = PWM_5;
    pwm_iocb.output.enable_mask = MASK_COMPARATOR_1;
    vos_dev_ioctl(hPwm, &pwm_iocb);
    // enable comparator 2 for PWM_2 and PWM_6
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_OUTPUT_TOGGLE_ENABLES;
    pwm_iocb.identifier.pwm_number = PWM_2;
    pwm_iocb.output.enable_mask = MASK_COMPARATOR_2;
    vos_dev_ioctl(hPwm, &pwm_iocb);
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_OUTPUT_TOGGLE_ENABLES;
    pwm_iocb.identifier.pwm_number = PWM_6;
    pwm_iocb.output.enable_mask = MASK_COMPARATOR_2;
    vos_dev_ioctl(hPwm, &pwm_iocb);
    // enable comparator 3 for PWM_3 and PWM_7
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_OUTPUT_TOGGLE_ENABLES;
    pwm_iocb.identifier.pwm_number = PWM_3;
    pwm_iocb.output.enable_mask = MASK_COMPARATOR_3;
    vos_dev_ioctl(hPwm, &pwm_iocb);
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_OUTPUT_TOGGLE_ENABLES;
    pwm_iocb.identifier.pwm_number = PWM_7;
    pwm_iocb.output.enable_mask = MASK_COMPARATOR_3;
    vos_dev_ioctl(hPwm, &pwm_iocb);

    // set initial state
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_INITIAL_STATE;
    pwm_iocb.output.init_state_mask = 0x00; // all low initially
    vos_dev_ioctl(hPwm, &pwm_iocb);

    // set restore state
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_RESTORE_INITIAL_STATE;
    pwm_iocb.output.restore_state_mask = (MASK_PWM_0 | MASK_PWM_1 | MASK_PWM_2 | MASK_PWM_3 | MASK_PWM_4 | MASK_PWM_5 | MASK_PWM_6 | MASK_PWM_7);    //all reset
    vos_dev_ioctl(hPwm, &pwm_iocb);

    // set mode to continuous
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_NUMBER_OF_CYCLES;
    pwm_iocb.output.mode = 0xB0; // 0 cycles -> continuous
    vos_dev_ioctl(hPwm, &pwm_iocb);

    // enable interrupt
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_ENABLE_INTERRUPT;
    vos_dev_ioctl(hPwm, &pwm_iocb);

    // enable output
    pwm_iocb.ioctl_code = VOS_IOCTL_PWM_ENABLE_OUTPUT;
    vos_dev_ioctl(hPwm, &pwm_iocb);

    do
    {
        // wait on interrupt
        // interrupt will be fired when No. of cycles counted
        pwm_iocb.ioctl_code = VOS_IOCTL_PWM_WAIT_ON_COMPLETE;
        vos_dev_ioctl(hPwm, &pwm_iocb);
        //-----------------------------
        // disable output
        pwm_iocb.ioctl_code = VOS_IOCTL_PWM_DISABLE_OUTPUT;
        vos_dev_ioctl(hPwm, &pwm_iocb);
        //-----------------------------
        // Comparator_0
        // check for change of direction required
        if (count0 == 0x00AF)
            direction0 = 0;
        if (count0 == 0x0001)
            direction0 = 1;
        // update comparator 0 value
        if (direction0 == 1)
            count0++; // increment value
        else
            count0--; // decrement value

        // set value for comparator_0
        pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COMPARATOR_VALUE;
        pwm_iocb.identifier.comparator_number = COMPARATOR_0;
        pwm_iocb.comparator.value = count0;
        vos_dev_ioctl(hPwm, &pwm_iocb);
        //-----------------------------
        // Comparator_1
        // check for change of direction required
        if (count1 == 0x00AF)
            direction1 = 0;
        if (count1 == 0x0001)
            direction1 = 1;
        // update comparator 0 value
        if (direction1 == 1)
            count1++; // increment value
        else
            count1--; // decrement value

        // set value for comparator_1
        pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COMPARATOR_VALUE;
        pwm_iocb.identifier.comparator_number = COMPARATOR_1;
        pwm_iocb.comparator.value = count1;
        vos_dev_ioctl(hPwm, &pwm_iocb);
        //-----------------------------
        // Comparator_2
        // check for change of direction required
        if (count2 == 0x00AF)
            direction2 = 0;
        if (count2 == 0x0001)
            direction2 = 1;
        // update comparator 0 value
        if (direction2 == 1)
            count2++; // increment value
        else
            count2--; // decrement value

        // set value for comparator_2
        pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COMPARATOR_VALUE;
        pwm_iocb.identifier.comparator_number = COMPARATOR_2;
        pwm_iocb.comparator.value = count2;
        vos_dev_ioctl(hPwm, &pwm_iocb);
        //-----------------------------
        // Comparator_3
        // check for change of direction required
        if (count3 == 0x00AF)
            direction3 = 0;
        if (count3 == 0x0001)
            direction3 = 1;
        // update comparator 0 value
        if (direction3 == 1)
            count3++; // increment value
        else
            count3--; // decrement value

        // set value for comparator_3
        pwm_iocb.ioctl_code = VOS_IOCTL_PWM_SET_COMPARATOR_VALUE;
        pwm_iocb.identifier.comparator_number = COMPARATOR_3;
        pwm_iocb.comparator.value = count3;
        vos_dev_ioctl(hPwm, &pwm_iocb);

        // enable output
        pwm_iocb.ioctl_code = VOS_IOCTL_PWM_ENABLE_OUTPUT;
        vos_dev_ioctl(hPwm, &pwm_iocb);
    }while(1);
}

